/* eslint-disable no-magic-numbers */
import { getReduxStoreWithActionsListener } from '@/utils/testing/getReduxStoreActionsListener';
import { mockNetworkResponse } from '@/utils/mockNetworkResponse';
import { apiPath } from '@/redux/apiPath';
import { HttpStatusCode } from 'axios';
import {
  ELEMENT_SEARCH_RESULT_MOCK_ALIAS,
  ELEMENT_SEARCH_RESULT_MOCK_REACTION,
} from '@/models/mocks/elementSearchResultMock';
import { waitFor } from '@testing-library/react';
import { FIRST_ARRAY_ELEMENT, SECOND_ARRAY_ELEMENT, SIZE_OF_EMPTY_ARRAY } from '@/constants/common';
import { onMapRightClick } from './onMapRightClick';
import * as handleDataReset from '../mapSingleClick/handleDataReset';
import * as handleSearchResultForRightClickAction from './handleSearchResultForRightClickAction';

jest.mock('./handleSearchResultForRightClickAction', () => ({
  __esModule: true,
  ...jest.requireActual('./handleSearchResultForRightClickAction'),
}));
jest.mock('../mapSingleClick/handleDataReset', () => ({
  __esModule: true,
  ...jest.requireActual('../mapSingleClick/handleDataReset'),
}));

const mockedAxiosOldClient = mockNetworkResponse();

const handleSearchResultForRightClickActionSpy = jest.spyOn(
  handleSearchResultForRightClickAction,
  'handleSearchResultForRightClickAction',
);
const handleDataResetSpy = jest.spyOn(handleDataReset, 'handleDataReset');

describe('onMapRightClick - util', () => {
  beforeEach(() => {
    jest.clearAllMocks();
    jest.resetAllMocks();
  });

  describe('when always', () => {
    const { store } = getReduxStoreWithActionsListener();
    const { dispatch } = store;
    const modelId = 1000;
    const mapSize = {
      width: 90,
      height: 90,
      tileSize: 256,
      minZoom: 2,
      maxZoom: 9,
    };
    const handler = onMapRightClick(mapSize, modelId, dispatch, false);
    const coordinate = [90, 90];
    const pixel = [250, 250];

    it('should fire data reset handler', async () => {
      await handler(coordinate, pixel);
      expect(handleDataResetSpy).toBeCalled();
    });
    it('should fire open context menu handler', async () => {
      const actions = store.getActions();
      expect(actions.length).toBeGreaterThan(SIZE_OF_EMPTY_ARRAY);
      expect(actions[FIRST_ARRAY_ELEMENT].type).toEqual('map/updateLastRightClick');
      expect(actions[SECOND_ARRAY_ELEMENT].type).toEqual('contextMenu/openContextMenu');
    });
  });

  describe('when searchResults are undefined', () => {
    const { store } = getReduxStoreWithActionsListener();
    const { dispatch } = store;
    const modelId = 1000;
    const mapSize = {
      width: 90,
      height: 90,
      tileSize: 256,
      minZoom: 2,
      maxZoom: 9,
    };
    const handler = onMapRightClick(mapSize, modelId, dispatch, false);
    const coordinate = [90, 90];
    const point = { x: 180.0008084837557, y: 179.99919151624428 };
    const pixel = [250, 250];

    mockedAxiosOldClient
      .onGet(apiPath.getSingleBioEntityContentsStringWithCoordinates(point, modelId))
      .reply(HttpStatusCode.Ok, undefined);

    it('does not fire search result action', async () => {
      await handler(coordinate, pixel);
      expect(handleSearchResultForRightClickActionSpy).not.toBeCalled();
    });
  });

  describe('when searchResults are valid', () => {
    describe('when results type is ALIAS', () => {
      const { store } = getReduxStoreWithActionsListener();
      const { dispatch } = store;
      const { modelId } = ELEMENT_SEARCH_RESULT_MOCK_ALIAS;
      const mapSize = {
        width: 270,
        height: 270,
        tileSize: 256,
        minZoom: 2,
        maxZoom: 9,
      };
      const coordinate = [270, 270];
      const point = { x: 540.0072763538013, y: 539.9927236461986 };
      const pixel = [250, 250];

      mockedAxiosOldClient
        .onGet(apiPath.getSingleBioEntityContentsStringWithCoordinates(point, modelId))
        .reply(HttpStatusCode.Ok, [ELEMENT_SEARCH_RESULT_MOCK_ALIAS]);

      it('does fire search result for right click action handler', async () => {
        const handler = onMapRightClick(mapSize, modelId, dispatch, false);
        await handler(coordinate, pixel);
        await waitFor(() => expect(handleSearchResultForRightClickActionSpy).toBeCalled());
      });
    });

    describe('when results type is REACTION', () => {
      const { store } = getReduxStoreWithActionsListener();
      const { dispatch } = store;
      const { modelId } = ELEMENT_SEARCH_RESULT_MOCK_REACTION;
      const mapSize = {
        width: 0,
        height: 0,
        tileSize: 256,
        minZoom: 2,
        maxZoom: 9,
      };
      const coordinate = [0, 0];
      const point = {
        x: 0,
        y: 0,
      };
      const pixel = [0, 0];

      mockedAxiosOldClient
        .onGet(apiPath.getSingleBioEntityContentsStringWithCoordinates(point, modelId))
        .reply(HttpStatusCode.Ok, [ELEMENT_SEARCH_RESULT_MOCK_REACTION]);

      it('does fire search result for right click action - handle reaction', async () => {
        const handler = onMapRightClick(mapSize, modelId, dispatch, false);
        await handler(coordinate, pixel);
        await waitFor(() => expect(handleSearchResultForRightClickActionSpy).toBeCalled());
      });
    });
  });
});
