import { PROJECT_ID } from '@/constants';
import { backgroundsFixture } from '@/models/fixtures/backgroundsFixture';
import {
  ToolkitStoreWithSingleSlice,
  createStoreInstanceUsingSliceReducer,
} from '@/utils/createStoreInstanceUsingSliceReducer';
import { mockNetworkResponse } from '@/utils/mockNetworkResponse';
import { HttpStatusCode } from 'axios';
import { unwrapResult } from '@reduxjs/toolkit';
import { apiPath } from '../apiPath';
import backgroundsReducer from './backgrounds.slice';
import { getAllBackgroundsByProjectId } from './backgrounds.thunks';
import { BackgroundsState } from './backgrounds.types';

jest.mock('../../utils/showToast');

const mockedAxiosClient = mockNetworkResponse();

const INITIAL_STATE: BackgroundsState = {
  data: [],
  loading: 'idle',
  error: { name: '', message: '' },
};

describe('backgrounds reducer', () => {
  let store = {} as ToolkitStoreWithSingleSlice<BackgroundsState>;
  beforeEach(() => {
    store = createStoreInstanceUsingSliceReducer('backgrounds', backgroundsReducer);
  });

  it('should match initial state', () => {
    const action = { type: 'unknown' };

    expect(backgroundsReducer(undefined, action)).toEqual(INITIAL_STATE);
  });
  it('should update store after succesfull getAllBackgroundsByProjectId query', async () => {
    mockedAxiosClient
      .onGet(apiPath.getAllBackgroundsByProjectIdQuery(PROJECT_ID))
      .reply(HttpStatusCode.Ok, backgroundsFixture);

    const { type } = await store.dispatch(getAllBackgroundsByProjectId(PROJECT_ID));
    const { data, loading, error } = store.getState().backgrounds;

    expect(type).toBe('backgrounds/getAllBackgroundsByProjectId/fulfilled');
    expect(loading).toEqual('succeeded');
    expect(error).toEqual({ message: '', name: '' });
    expect(data).toEqual(backgroundsFixture);
  });

  it('should update store after failed getAllBackgroundsByProjectId query', async () => {
    mockedAxiosClient
      .onGet(apiPath.getAllBackgroundsByProjectIdQuery(PROJECT_ID))
      .reply(HttpStatusCode.NotFound, []);

    const action = await store.dispatch(getAllBackgroundsByProjectId(PROJECT_ID));
    const { data, loading, error } = store.getState().backgrounds;

    expect(action.type).toBe('backgrounds/getAllBackgroundsByProjectId/rejected');
    expect(loading).toEqual('failed');
    expect(error).toEqual({ message: '', name: '' });
    expect(data).toEqual([]);
    expect(() => unwrapResult(action)).toThrow(
      "Failed to fetch backgrounds: The page you're looking for doesn't exist. Please verify the URL and try again.",
    );
  });

  it('should update store on loading getAllBackgroundsByProjectId query', async () => {
    mockedAxiosClient
      .onGet(apiPath.getAllBackgroundsByProjectIdQuery(PROJECT_ID))
      .reply(HttpStatusCode.Ok, backgroundsFixture);

    const actionPromise = store.dispatch(getAllBackgroundsByProjectId(PROJECT_ID));

    const { data, loading } = store.getState().backgrounds;
    expect(data).toEqual([]);
    expect(loading).toEqual('pending');

    actionPromise.then(() => {
      const { data: dataPromiseFulfilled, loading: promiseFulfilled } =
        store.getState().backgrounds;

      expect(dataPromiseFulfilled).toEqual(backgroundsFixture);
      expect(promiseFulfilled).toEqual('succeeded');
    });
  });
});
